################################################
#       zABBER Configuration Handle File       #
################################################
#                                              #
#   This file handles all things about         #
#   configuration and config window.           #
#                                              #
################################################

source ./INIparse/iniparse.tcl
source ./scripts.tcl

set cfile "config.ini"
set config(list) {Happy Programming Playing Eating Normal Away Sad Angry}
set config(old)  {Happy}
set config(now)  ""

proc loadconfig {} {
global jabber cfile tcl_platform

set jabber(username) [readini $cfile login]
set jabber(pass) [readini $cfile pass]
set jabber(nick) [readini $cfile nick]
set jabber(server) [readini $cfile server]

set jabber(msgs,happy) [cfgdecrypt [readini $cfile msgs_happy]]
set jabber(msgs,programming) [cfgdecrypt [readini $cfile msgs_programming]]
set jabber(msgs,playing) [cfgdecrypt [readini $cfile msgs_playing]]
set jabber(msgs,eating) [cfgdecrypt [readini $cfile msgs_eating]]
set jabber(msgs,normal) [cfgdecrypt [readini $cfile msgs_normal]]
set jabber(msgs,away) [cfgdecrypt [readini $cfile msgs_away]]
set jabber(msgs,sad) [cfgdecrypt [readini $cfile msgs_sad]]
set jabber(msgs,angry) [cfgdecrypt [readini $cfile msgs_angry]]

set jabber(misc,beeponmsg) [readini $cfile beeponmsg]
set jabber(misc,beeponerr) [readini $cfile beeponerr]
set jabber(misc,showpasswin)   [readini $cfile showpasswin]
if {$tcl_platform(platform)!="windows"} {
	set jabber(misc,browser) [readini $cfile browser]
    } else {set jabber(misc,browser) "NOT NEEDED UNDER WINDOWS"}

set jabber(win,geom_mainwin) [readini $cfile geom_mainwin]
set jabber(win,geom_sendwin) [readini $cfile geom_sendwin]
set jabber(win,geom_receiwin) [readini $cfile geom_receiwin]

loadscripts
}

proc save_geom_mainwin {} {
global cfile
writeini $cfile geom_mainwin [wm geometry .win]
}

proc save_geom_sendwin {w} {
global cfile jabber
set jabber(win,geom_sendwin) [wm geometry $w]
writeini $cfile geom_sendwin $jabber(win,geom_sendwin)
}

proc save_geom_receiwin {w} {
global cfile jabber
set jabber(win,geom_receiwin) [wm geometry $w]
writeini $cfile geom_receiwin $jabber(win,geom_receiwin)
}

proc configwindow {} {
global cfile config jabber script tcl_platform

  set config(msgs,Happy)       $jabber(msgs,happy)
  set config(msgs,Programming) $jabber(msgs,programming)
  set config(msgs,Playing)     $jabber(msgs,playing)
  set config(msgs,Eating)      $jabber(msgs,eating)
  set config(msgs,Normal)      $jabber(msgs,normal)
  set config(msgs,Away)        $jabber(msgs,away)
  set config(msgs,Sad)         $jabber(msgs,sad)
  set config(msgs,Angry)       $jabber(msgs,angry)

  set config(3,beeponmsg)      $jabber(misc,beeponmsg)
  set config(3,beeponerr)      $jabber(misc,beeponerr)
  set config(3,showpasswin)    $jabber(misc,showpasswin)
  set config(4,list)           ""

    set base ".confwin"

    set x [expr ([winfo screenwidth .]-300)/2]
    set y [expr ([winfo screenheight .]-250)/2]

    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 300x250+$x+$y
    wm overrideredirect $base 0
    wm resizable $base 0 0
    wm maxsize $base 400 300
    wm deiconify $base
    wm transient $base .win
    wm title $base "Configure"
    grab set $base
    focus $base

    Notebook:create $base.n -pages {Login Messages Misc Scripts} -width 1 -height 1
    set f1 [Notebook:frame $base.n "Login"]
    set f2 [Notebook:frame $base.n "Messages"]
    set f3 [Notebook:frame $base.n "Misc"]
    set f4 [Notebook:frame $base.n "Scripts"]

    Notebook:pageconfig $base.n Login -onexit {{
			if {[.confwin.n.f0.servname get]==""} {tk_messageBox -parent .confwin -title "Error" -message "You must enter a server address!"; return 0}
			if {[.confwin.n.f0.logname get]==""} {tk_messageBox -parent .confwin -title "Error" -message "You must enter a login name!"; return 0}
			if {[.confwin.n.f0.password get]==""} {tk_messageBox -parent .confwin -title "Error" -message "You must enter a password!"; return 0}
			}}
    frame $f1.f -relief sunken -borderwidth 1
    frame $f2.f -relief sunken -borderwidth 1
    frame $f3.f -relief sunken -borderwidth 1
    frame $f4.f -relief sunken -borderwidth 1
    button $f1.f.b1 -borderwidth 1 -text Ok -command "config:save $f1 $f2 $f3; destroy $base"
    button $f2.f.b1 -borderwidth 1 -text Ok -command "config:save $f1 $f2 $f3; destroy $base"
    button $f3.f.b1 -borderwidth 1 -text Ok -command "config:save $f1 $f2 $f3; destroy $base"
    button $f4.f.b1 -borderwidth 1 -text Ok -command "config:save $f1 $f2 $f3; destroy $base"
    button $f1.f.b2 -borderwidth 1 -text Cancel -command "destroy $base"
    button $f2.f.b2 -borderwidth 1 -text Cancel -command "destroy $base"
    button $f3.f.b2 -borderwidth 1 -text Cancel -command "destroy $base"
    button $f4.f.b2 -borderwidth 1 -text Cancel -command "destroy $base"
    button $f1.f.b3 -borderwidth 1 -text Help
    button $f2.f.b3 -borderwidth 1 -text Help
    button $f3.f.b3 -borderwidth 1 -text Help
    button $f4.f.b3 -borderwidth 1 -text Help

    pack $base.n -expand 1 -fill both -ipadx 10 -ipady 10
    place $f1.f -x 200 -y 0 -width 66 -height 62
    place $f2.f -x 200 -y 0 -width 66 -height 62
    place $f3.f -x 200 -y 0 -width 66 -height 62
    place $f4.f -x 200 -y 0 -width 66 -height 62
    place $f1.f.b1 -x 0 -y 0 -width 64 -height 20
    place $f2.f.b1 -x 0 -y 0 -width 64 -height 20
    place $f3.f.b1 -x 0 -y 0 -width 64 -height 20
    place $f4.f.b1 -x 0 -y 0 -width 64 -height 20
    place $f1.f.b2 -x 0 -y 20 -width 64 -height 20
    place $f2.f.b2 -x 0 -y 20 -width 64 -height 20
    place $f3.f.b2 -x 0 -y 20 -width 64 -height 20
    place $f4.f.b2 -x 0 -y 20 -width 64 -height 20
    place $f1.f.b3 -x 0 -y 40 -width 64 -height 20
    place $f2.f.b3 -x 0 -y 40 -width 64 -height 20
    place $f3.f.b3 -x 0 -y 40 -width 64 -height 20
    place $f4.f.b3 -x 0 -y 40 -width 64 -height 20


    ################
    # Page 1
    ################

    entry $f1.logname -width 0 -background #ffffff
    entry $f1.servname -width 0 -background #ffffff
    entry $f1.password -width 0 -show "*" -background #ffffff
    entry $f1.nickname -width 0 -background #ffffff
    message $f1.mes1 -text "Please enter your\naccount information" -justify center -aspect 5000
    message $f1.mes2 -text "Username:" -aspect 5000
    message $f1.mes3 -text "Server:" -aspect 5000
    message $f1.mes4 -text "@" -aspect 5000
    message $f1.mes5 -text "Password:" -aspect 5000
    message $f1.mes6 -text "Nickname:" -aspect 5000

    place $f1.logname -x 0 -y 80 -width 96 -height 19
    place $f1.servname -x 110 -y 80 -width 146 -height 19
    place $f1.password -x 70 -y 110 -width 186 -height 19
    place $f1.nickname -x 70 -y 135 -width 186 -height 19
    place $f1.mes1 -x 25 -y 15
    place $f1.mes2 -x -5 -y 65 -width 65 -height 11
    place $f1.mes3 -x 105 -y 65 -width 48 -height 11
    place $f1.mes4 -x 95 -y 80 -width 15 -height 21
    place $f1.mes5 -x -5 -y 110
    place $f1.mes6 -x -5 -y 135

    $f1.logname insert 0 $jabber(username)
    $f1.servname insert 0 $jabber(server)
    $f1.password insert 0 $jabber(pass)
    $f1.nickname insert 0 $jabber(nick)

    ################
    # Page 2
    ################

    text     $f2.t     -width 1 -height 1 -background #ffffff
    message  $f2.mes   -aspect 5000 -text {Select message to customize:}
    $f2.t insert end $config(msgs,Happy)
    
    combobox $f2.combo -width 1 -value [lindex $config(list) 0] -textvariable config(now) -editable 0 -background #ffffff \
	-command {
		if {$config(now)==""} {return}
		set abc [.confwin.n.f1.t get 1.0 end]
		set config(msgs,$config(old)) [string range $abc 0 [expr [string length $abc]-2]]
		.confwin.n.f1.t delete 1.0 end
		.confwin.n.f1.t insert end $config(msgs,$config(now))
		set config(old) $config(now)
		unset abc
	}

    foreach abc $config(list) {$f2.combo list insert end $abc}

    place $f2.t     -x 0 -y 75 -width 266 -height 108
    place $f2.mes   -x 0 -y 5 -width 158 -height 21
    place $f2.combo -x 25 -y 35 -width 96 -height 19

    ################
    # Page 3
    ################

    checkbutton $f3.beeponmsg   -indicatoron 1 -variable config(3,beeponmsg) -text "Beep on message"
    checkbutton $f3.beeponerr   -indicatoron 1 -variable config(3,beeponerr) -text "Beep on error"
    checkbutton $f3.showpasswin -indicatoron 1 -variable config(3,showpasswin)   -text "Set password on connect"
    frame       $f3.f1          -borderwidth 2 -relief groove
    message     $f3.mes         -aspect 5000 -text "Browser:"
    entry       $f3.f1.ent      -background #ffffff
    $f3.f1.ent insert end $jabber(misc,browser)

    set types {{{All Files} {*}}}
    set first "set f3 .confwin.n.f2; set types \"$types\"; set base $base\n"
    set rest  {set a [tk_getOpenFile -filetypes $types -parent $base -initialdir / -title "Select Browser"]
	       if {$a!=""} {$f3.f1.ent delete 0 end
			    $f3.f1.ent insert 0 $a}
	       unset a types f3 base}
    button      $f3.f1.but    -text "Select" -command "$first$rest"

#
# Because we would use the "start" command instead of running
# browser directly for providing speed and compability, we
# disable show "Select Browser" things.
#
    if {$tcl_platform(platform)=="windows"} {
 	$f3.mes configure -foreground #808080
	$f3.f1.ent configure -state disabled -background #c0c0c0
	$f3.f1.but configure -state disabled
    }

    place $f3.beeponmsg   -x 0 -y 0 -width [winfo reqwidth $f3.beeponmsg] -height 20
    place $f3.beeponerr   -x 0 -y 20 -width [winfo reqwidth $f3.beeponerr] -height 20
    place $f3.showpasswin -x 0 -y 40 -width [winfo reqwidth $f3.showpasswin] -height 20
    place $f3.f1          -x 0 -y 143 -width 266 -height 40
    place $f3.mes         -x 5 -y 135 -width [winfo reqwidth $f3.mes] -height 15
    place $f3.f1.ent      -x 8 -y 9 -width 185 -height [winfo reqheight $f3.f1.ent]
    place $f3.f1.but      -x 203 -y 5 -width 50 -height [winfo reqheight $f3.f1.but]

    ################
    # Page 4
    ################

    set first "set f4 $f4;"
    set rest  {
	set file [selectinf]
	grab set .confwin
	focus .confwin
	if {$file!=""} {
		if {[set temp [script:handle:inf $file]]!=-1} {
			set i [readini "Scripts/scripts.ini" num]
			incr i
			writeini "Scripts/scripts.ini" num $i
			writeini "Scripts/scripts.ini" "script$i" "$file"

			lappend script(files)        $file
			set     script(files,$file)  $temp
			set     script(interp,$file) ""

			set a ""
			foreach a "[lindex $temp 0]" {.popup_group.script add command -label "[lindex $a 0]" -command "script:start:groupmenu \"$file\" \"[lindex $a 1]\""}
			foreach a "[lindex $temp 1]" {.popup_user.script add command -label "[lindex $a 0]" -command "script:start:usermenu \"$file\" \"[lindex $a 1]\""}
			foreach a "[lindex $temp 2]" {.popup_nick.script add command -label "[lindex $a 0]" -command "script:start:nickmenu \"$file\" \"[lindex $a 1]\""}
			unset i temp a
		} else {tk_messageBox -message "Error in script!" -icon error}

		$f4.f1.list delete 0 end
		set config(4,list) ""
		foreach file $script(files) {
			$f4.f1.list insert end "[lindex $script(files,$file) 3]"
			lappend config(4,list) $file
		}
		unset f4
	}
	unset file
    }

    frame   $f4.f1 -relief sunken -borderwidth 2
    listbox $f4.f1.list -relief flat -borderwidth 0 -background #ffffff -yscrollcommand "$f4.f1.scr set"
    if {$tcl_platform(platform)=="unix"} {$f4.f1.list configure -font {{Helvetica} 12}}
    scrollbar $f4.f1.scr -command "$f4.f1.list yview"
    button  $f4.b1 -text "Add" -borderwidth 1 -command "$first$rest"
    button  $f4.b2 -text "Remove" -borderwidth 1 -command "config:4:remove $f4"
    button  $f4.b3 -text "More..." -borderwidth 1

    foreach file $script(files) {
	$f4.f1.list insert end "[lindex $script(files,$file) 3]"
	lappend config(4,list) $file
    }

    place $f4.f1   -x 0 -y 0 -width 180 -height 184
    pack  $f4.f1.list -side left -fill both -expand 1
    pack  $f4.f1.scr  -side left -fill y
    place $f4.b1   -x 201 -y 124 -width 64 -height 20
    place $f4.b2   -x 201 -y 144 -width 64 -height 20
    place $f4.b3   -x 201 -y 164 -width 64 -height 20

}

proc config:save {f1 f2 f3} {
global cfile config jabber tcl_platform

  ##################
  # Save Page1
  ##################

  if {[set a1 [$f1.servname get]]==""} {tk_messageBox -parent .confwin -title "Error" -message "You must enter a server address!"; return}
  if {[set a2 [$f1.logname  get]]==""} {tk_messageBox -parent .confwin -title "Error" -message "You must enter a login name!"; return}
  if {[set a3 [$f1.password get]]==""} {tk_messageBox -parent .confwin -title "Error" -message "You must enter a password!"; return}
  set a4 [$f1.nickname get]

  if {$a1!=$jabber(server)}   {set jabber(server)   $a1; writeini $cfile server $a1}
  if {$a2!=$jabber(username)} {set jabber(username) $a2; writeini $cfile login  $a2}
  if {$a3!=$jabber(pass)}     {set jabber(pass)     $a3; writeini $cfile pass   $a3}
  if {$a4!=$jabber(nick)}     {set jabber(nick)     $a4; writeini $cfile nick   $a4}

  ##################
  # Save Page2
  ##################

  set abc [$f2.t get 1.0 end]
  set config(msgs,$config(now)) [string range $abc 0 [expr [string length $abc]-2]]

  if {$jabber(msgs,happy)!=$config(msgs,Happy)} {set jabber(msgs,happy) $config(msgs,Happy); writeini $cfile msgs_happy [cfgcrypt $jabber(msgs,happy)]}
  if {$jabber(msgs,programming)!=$config(msgs,Programming)} {set jabber(msgs,programming) $config(msgs,Programming); writeini $cfile msgs_programming [cfgcrypt $jabber(msgs,programming)]}
  if {$jabber(msgs,playing)!=$config(msgs,Playing)} {set jabber(msgs,playing) $config(msgs,Playing); writeini $cfile msgs_playing [cfgcrypt $jabber(msgs,playing)]}
  if {$jabber(msgs,eating)!=$config(msgs,Eating)} {set jabber(msgs,eating) $config(msgs,Eating); writeini $cfile msgs_eating [cfgcrypt $jabber(msgs,eating)]}
  if {$jabber(msgs,normal)!=$config(msgs,Normal)} {set jabber(msgs,normal) $config(msgs,Normal); writeini $cfile msgs_normal [cfgcrypt $jabber(msgs,normal)]}
  if {$jabber(msgs,away)!=$config(msgs,Away)} {set jabber(msgs,away) $config(msgs,Away); writeini $cfile msgs_away [cfgcrypt $jabber(msgs,away)]}
  if {$jabber(msgs,sad)!=$config(msgs,Sad)} {set jabber(msgs,sad) $config(msgs,Sad); writeini $cfile msgs_sad [cfgcrypt $jabber(msgs,sad)]}
  if {$jabber(msgs,angry)!=$config(msgs,Angry)} {set jabber(msgs,angry) $config(msgs,Angry); writeini $cfile msgs_angry [cfgcrypt $jabber(msgs,angry)]}

  ##################
  # Save Page3
  ##################

  if {$config(3,beeponmsg)!=$jabber(misc,beeponmsg)} {set jabber(misc,beeponmsg) $config(3,beeponmsg); writeini $cfile beeponmsg $jabber(misc,beeponmsg)}
  if {$config(3,beeponerr)!=$jabber(misc,beeponerr)} {set jabber(misc,beeponerr) $config(3,beeponerr); writeini $cfile beeponerr $jabber(misc,beeponerr)}
  if {$config(3,showpasswin)!=$jabber(misc,showpasswin)} {set jabber(misc,showpasswin) $config(3,showpasswin); writeini $cfile showpasswin $jabber(misc,showpasswin)}
  if {$tcl_platform(platform)!="windows" && [set a [$f3.f1.ent get]]!=$jabber(misc,browser)} {set jabber(misc,browser) $a; writeini $cfile browser $a; unset a}

}

proc config:4:remove {f4} {
global script config

  if {[set selected [$f4.f1.list curselection]]==""} {return}
  set selected [lindex $selected 0]

  set file_ [lindex $config(4,list) $selected]

  set found 0
  set num [readini "Scripts/scripts.ini" "num"]
  if {!$num} {return}    ;#Return if $num is 0.
  for {set i 1} {$i<=$num} {incr i} {
	if {[readini "Scripts/scripts.ini" "script$i"]==$file_} {set found 1}
  }
  if {!$found} {return}


  if {[tk_messageBox -icon question -parent .confwin -type yesno -title "Delete Script?" \
       -message "Are you sure you want to remove this script?"]=="no"} {return}

  set num [readini "Scripts/scripts.ini" "num"]
  if {$num==0} {return}
  writeini "Scripts/scripts.ini" "num" [expr $num-1]
  for {set i 1} {$i<=$num} {incr i} {
	set file [readini "Scripts/scripts.ini" "script$i"]
	if {$file==$file_} {
		  removeini "Scripts/scripts.ini" "script$i"
		  for  {set i1 [expr $i+1]} {$i1<=$num} {incr i1} {
			set file [readini "Scripts/scripts.ini" "script$i"]
			renameini "Scripts/scripts.ini" "script$i1" "script[expr $i1-1]"
		       }
		  break
	}
  }

  tk_messageBox -message "You should restart zABBER for changes to take effect!"
}

#
# Proc to crypt "enter" chars used in cfg file.
#
proc cfgcrypt {text} {
set retext ""
set cry1 {_;}
set cry2 {__}

  for {set x "0"} {$x<[string length $text]} {incr x} {
    set char [string index $text $x]

    switch -- $char {
      "\n" {set retext $retext$cry1}
      "_"  {set retext $retext$cry2}
      default {set retext $retext$char}
      }
    }
  return $retext
}

#
# Proc to decrypt "enter" chars used in cfg file.
#
proc cfgdecrypt {text} {
set retext ""
set char ""
set cry1 {_;}
set cry_1 "\n"
set cry2 {__}
set cry_2 "_"

  for {set x "0"} {$x<[string length $text]} {incr x} {
    set char $char[string index $text $x]

    if { $char==$cry1 } {set retext $retext$cry_1; set char ""} \
      elseif { $char==$cry2 } {set retext $retext$cry_2; set char ""} \
        \
      elseif { "$char[rtrimleft $cry1 $char]"==$cry1 } {} \
      elseif { "$char[rtrimleft $cry2 $char]"==$cry2 } {} \
      else { set retext $retext$char; set char "" }
    }
  return $retext
}

proc selectinf {} {
global temp tcl_platform
  set temp 0
  set base ".selectinf"

  toplevel $base -class Toplevel
  wm focusmodel $base passive
  wm geometry $base 150x250
  wm overrideredirect $base 0
  wm resizable $base 0 0
  wm deiconify $base
  wm transient $base .
  wm title $base "Select Script"
  wm protocol $base WM_DELETE_WINDOW "set temp -1"
  grab set $base
  focus $base
  
  frame $base.f0 -borderwidth 2 -relief sunken
  listbox $base.f0.l -borderwidth 0 -relief flat -yscrollcommand "$base.f0.s set" -background #ffffff
  scrollbar $base.f0.s -command "$base.f0.l yview"
  frame $base.f1 -borderwidth 0 -relief flat
  button $base.f1.ok -text "Select" -command "if \{\[$base.f0.l curselection\]!=\"\"\} \{set temp 1\}"
  button $base.f1.cancel -text "Cancel" -command "set temp -1"

  pack $base.f0 -fill both -expand 1
  pack $base.f0.l -side left -fill both -expand 1
  pack $base.f0.s -side left -fill y
  pack $base.f1 -fill x
  pack $base.f1.ok -side left -fill x -expand 1
  pack $base.f1.cancel -side left -fill x -expand 1


  #
  # Fill listbox
  #
  cd "Scripts"

  set a [glob -nocomplain -- "*.inf"]
  cd ".."

  if {$tcl_platform(platform)=="windows"} {set a [string tolower $a]}

  foreach b $a {$base.f0.l insert end $b}
  set a ""; set b ""

  #
  # Wait user
  #
  tkwait variable temp

  if {$temp==-1} {destroy $base; return ""}

  set a [$base.f0.l get [$base.f0.l curselection]]
  destroy $base
  return [string trimright $a ".inf"]
}
